@@ -122,12 +122,6 @@ struct userdata {
     bool use_ucm;
     pa_alsa_ucm_config ucm;
 
-    /* hooks for modifier action */
-    pa_hook_slot
-        *sink_input_put_hook_slot,
-        *source_output_put_hook_slot,
-        *sink_input_unlink_hook_slot,
-        *source_output_unlink_hook_slot;
 };
 
 struct profile_data {
@@ -384,16 +378,17 @@ static int report_jack_state(snd_mixer_elem_t *melem, unsigned int mask) {
 
     PA_HASHMAP_FOREACH(jack, u->jacks, state)
         if (jack->melem == melem) {
-            jack->plugged_in = plugged_in;
+            pa_alsa_jack_set_plugged_in(jack, plugged_in);
+
             if (u->use_ucm) {
-                pa_assert(u->card->ports);
-                port = pa_hashmap_get(u->card->ports, jack->name);
-                pa_assert(port);
-            }
-            else {
-                pa_assert(jack->path);
-                pa_assert_se(port = jack->path->port);
+                /* When using UCM, pa_alsa_jack_set_plugged_in() maps the jack
+                 * state to port availability. */
+                continue;
             }
+
+            /* When not using UCM, we have to do the jack state -> port
+             * availability mapping ourselves. */
+            pa_assert_se(port = jack->path->port);
             report_port_state(port, u);
         }
     return 0;
@@ -521,7 +516,7 @@ static void init_jacks(struct userdata *u) {
             jack->melem = pa_alsa_mixer_find(u->mixer_handle, jack->alsa_name, 0);
             if (!jack->melem) {
                 pa_log_warn("Jack '%s' seems to have disappeared.", jack->alsa_name);
-                jack->has_control = false;
+                pa_alsa_jack_set_has_control(jack, false);
                 continue;
             }
             snd_mixer_elem_set_callback(jack->melem, report_jack_state);
@@ -677,16 +672,16 @@ int pa__init(pa_module *m) {
 
         /* hook start of sink input/source output to enable modifiers */
         /* A little bit later than module-role-cork */
-        u->sink_input_put_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE+10,
+        pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_PUT], PA_HOOK_LATE+10,
                 (pa_hook_cb_t) sink_input_put_hook_callback, u);
-        u->source_output_put_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE+10,
+        pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_PUT], PA_HOOK_LATE+10,
                 (pa_hook_cb_t) source_output_put_hook_callback, u);
 
         /* hook end of sink input/source output to disable modifiers */
         /* A little bit later than module-role-cork */
-        u->sink_input_unlink_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE+10,
+        pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SINK_INPUT_UNLINK], PA_HOOK_LATE+10,
                 (pa_hook_cb_t) sink_input_unlink_hook_callback, u);
-        u->source_output_unlink_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE+10,
+        pa_module_hook_connect(m, &m->core->hooks[PA_CORE_HOOK_SOURCE_OUTPUT_UNLINK], PA_HOOK_LATE+10,
                 (pa_hook_cb_t) source_output_unlink_hook_callback, u);
     }
     else {
@@ -820,18 +815,6 @@ void pa__done(pa_module*m) {
     if (!(u = m->userdata))
         goto finish;
 
-    if (u->sink_input_put_hook_slot)
-        pa_hook_slot_free(u->sink_input_put_hook_slot);
-
-    if (u->sink_input_unlink_hook_slot)
-        pa_hook_slot_free(u->sink_input_unlink_hook_slot);
-
-    if (u->source_output_put_hook_slot)
-        pa_hook_slot_free(u->source_output_put_hook_slot);
-
-    if (u->source_output_unlink_hook_slot)
-        pa_hook_slot_free(u->source_output_unlink_hook_slot);
-
     if (u->mixer_fdl)
         pa_alsa_fdlist_free(u->mixer_fdl);
     if (u->mixer_handle)
