@@ -212,7 +212,7 @@ int pa_client_conf_load_cookie(pa_client_conf *c, uint8_t *cookie, size_t cookie
     if (r >= 0)
         return 0;
 
-    if (pa_append_to_home_dir(PA_NATIVE_COOKIE_FILE_FALLBACK, &fallback_path) > 0) {
+    if (pa_append_to_home_dir(PA_NATIVE_COOKIE_FILE_FALLBACK, &fallback_path) >= 0) {
         r = pa_authkey_load(fallback_path, false, cookie, cookie_length);
         pa_xfree(fallback_path);
         if (r >= 0)
