@@ -15,9 +15,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -97,7 +95,7 @@ pa_threaded_mainloop *pa_threaded_mainloop_new(void) {
 
     pa_init_i18n();
 
-    m = pa_xnew(pa_threaded_mainloop, 1);
+    m = pa_xnew0(pa_threaded_mainloop, 1);
 
     if (!(m->real_mainloop = pa_mainloop_new())) {
         pa_xfree(m);
@@ -107,14 +105,9 @@ pa_threaded_mainloop *pa_threaded_mainloop_new(void) {
     m->mutex = pa_mutex_new(true, true);
     m->cond = pa_cond_new();
     m->accept_cond = pa_cond_new();
-    m->thread = NULL;
-    m->name = NULL;
 
     pa_mainloop_set_poll_func(m->real_mainloop, poll_func, m->mutex);
 
-    m->n_waiting = 0;
-    m->n_waiting_for_accept = 0;
-
     return m;
 }
 
