@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -130,9 +128,9 @@ static int parse_line(pa_config_parser_state *state) {
                 return -1;
             }
 
-            state->in_proplist = TRUE;
+            state->in_proplist = true;
         } else
-            state->in_proplist = FALSE;
+            state->in_proplist = false;
 
         return 0;
     }
@@ -157,7 +155,7 @@ static int parse_line(pa_config_parser_state *state) {
 /* Go through the file and parse each line */
 int pa_config_parse(const char *filename, FILE *f, const pa_config_item *t, pa_proplist *proplist, void *userdata) {
     int r = -1;
-    pa_bool_t do_close = !f;
+    bool do_close = !f;
     pa_config_parser_state state;
 
     pa_assert(filename);
@@ -175,6 +173,7 @@ int pa_config_parse(const char *filename, FILE *f, const pa_config_item *t, pa_p
         pa_log_warn("Failed to open configuration file '%s': %s", filename, pa_cstrerror(errno));
         goto finish;
     }
+    pa_log_debug("Parsing configuration file '%s'", filename);
 
     state.filename = filename;
     state.item_table = t;
@@ -268,7 +267,7 @@ int pa_config_parse_size(pa_config_parser_state *state) {
 
 int pa_config_parse_bool(pa_config_parser_state *state) {
     int k;
-    pa_bool_t *b;
+    bool *b;
 
     pa_assert(state);
 
@@ -286,7 +285,7 @@ int pa_config_parse_bool(pa_config_parser_state *state) {
 
 int pa_config_parse_not_bool(pa_config_parser_state *state) {
     int k;
-    pa_bool_t *b;
+    bool *b;
 
     pa_assert(state);
 
