@@ -14,9 +14,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #ifdef HAVE_CONFIG_H
@@ -418,9 +416,6 @@ int pa_modargs_get_alternate_sample_rate(pa_modargs *ma, uint32_t *alternate_rat
         !pa_sample_rate_valid(*alternate_rate))
         return -1;
 
-    if (!((rate_local % 4000 == 0) || (rate_local % 11025 == 0)))
-        return -1;
-
     *alternate_rate = rate_local;
 
     return 0;
