@@ -100,6 +100,13 @@ bool pa_module_exists(const char *name) {
     return false;
 }
 
+void pa_module_hook_connect(pa_module *m, pa_hook *hook, pa_hook_priority_t prio, pa_hook_cb_t cb, void *data) {
+    pa_assert(m);
+    pa_assert(hook);
+    pa_assert(m->hooks);
+    pa_dynarray_append(m->hooks, pa_hook_connect(hook, prio, cb, data));
+}
+
 pa_module* pa_module_load(pa_core *c, const char *name, const char *argument) {
     pa_module *m = NULL;
     bool (*load_once)(void);
@@ -117,6 +124,7 @@ pa_module* pa_module_load(pa_core *c, const char *name, const char *argument) {
     m->argument = pa_xstrdup(argument);
     m->load_once = false;
     m->proplist = pa_proplist_new();
+    m->hooks = pa_dynarray_new((pa_free_cb_t) pa_hook_slot_free);
     m->index = PA_IDXSET_INVALID;
 
     if (!(m->dl = lt_dlopenext(name))) {
@@ -192,6 +200,8 @@ pa_module* pa_module_load(pa_core *c, const char *name, const char *argument) {
         pa_modinfo_free(mi);
     }
 
+    pa_hook_fire(&m->core->hooks[PA_CORE_HOOK_MODULE_NEW], m);
+
     return m;
 
 fail:
@@ -200,6 +210,9 @@ fail:
         if (m->index != PA_IDXSET_INVALID)
             pa_idxset_remove_by_index(c->modules, m->index);
 
+        if (m->hooks)
+            pa_dynarray_free(m->hooks);
+
         if (m->proplist)
             pa_proplist_free(m->proplist);
 
@@ -220,6 +233,12 @@ static void pa_module_free(pa_module *m) {
     pa_assert(m->core);
 
     pa_log_info("Unloading \"%s\" (index: #%u).", m->name, m->index);
+    pa_hook_fire(&m->core->hooks[PA_CORE_HOOK_MODULE_UNLINK], m);
+
+    if (m->hooks) {
+       pa_dynarray_free(m->hooks);
+       m->hooks = NULL;
+    }
 
     if (m->done)
         m->done(m);
@@ -229,6 +248,8 @@ static void pa_module_free(pa_module *m) {
 
     lt_dlclose(m->dl);
 
+    pa_hashmap_remove(m->core->modules_pending_unload, m);
+
     pa_log_info("Unloaded \"%s\" (index: #%u).", m->name, m->index);
 
     pa_subscription_post(m->core, PA_SUBSCRIPTION_EVENT_MODULE|PA_SUBSCRIPTION_EVENT_REMOVE, m->index);
@@ -245,8 +266,6 @@ void pa_module_unload(pa_core *c, pa_module *m, bool force) {
     if (m->core->disallow_module_loading && !force)
         return;
 
-    pa_hashmap_remove(c->modules_pending_unload, m);
-
     if (!(m = pa_idxset_remove_by_data(c->modules, m, NULL)))
         return;
 
@@ -294,12 +313,17 @@ void pa_module_unload_all(pa_core *c) {
     pa_xfree(indices);
 
     /* Just in case module unloading caused more modules to load */
+    PA_IDXSET_FOREACH(m, c->modules, state)
+        pa_log_warn("After module unload, module '%s' was still loaded!", m->name);
+    c->disallow_module_loading = 1;
     pa_idxset_remove_all(c->modules, (pa_free_cb_t) pa_module_free);
+    pa_assert(pa_idxset_isempty(c->modules));
 
     if (c->module_defer_unload_event) {
         c->mainloop->defer_free(c->module_defer_unload_event);
         c->module_defer_unload_event = NULL;
     }
+    pa_assert(pa_hashmap_isempty(c->modules_pending_unload));
 }
 
 static void defer_cb(pa_mainloop_api*api, pa_defer_event *e, void *userdata) {
@@ -354,4 +378,5 @@ void pa_module_update_proplist(pa_module *m, pa_update_mode_t mode, pa_proplist
         pa_proplist_update(m->proplist, mode, p);
 
     pa_subscription_post(m->core, PA_SUBSCRIPTION_EVENT_MODULE|PA_SUBSCRIPTION_EVENT_CHANGE, m->index);
+    pa_hook_fire(&m->core->hooks[PA_CORE_HOOK_MODULE_PROPLIST_CHANGED], m);
 }
